@@ -1,6 +1,6 @@
 package Sort::Key;
 
-our $VERSION = '1.27';
+our $VERSION = '1.31';
 
 use 5.006;
 
@@ -8,6 +8,8 @@ use strict;
 use warnings;
 use Carp;
 
+use Sort::Key::Types;
+
 require Exporter;
 
 our @ISA = qw(Exporter);
@@ -33,156 +35,18 @@ our @EXPORT_OK = qw( nsort nsort_inplace
 require XSLoader;
 XSLoader::load('Sort::Key', $VERSION);
 
-our $DEBUG;
-$DEBUG ||= 0;
-
-# this hash is also used from Sort::Key::Multi to find out which
-# letters can be used as types:
-our %mktypes = ( s => 0,
-                 l => 1,
-                 n => 2,
-                 i => 3,
-                 u => 4 );
-
-sub _mks2n {
-    if (my ($rev, $key)=$_[0]=~/^([-+]?)(.)$/) {
-	exists $mktypes{$key}
-	    or croak "invalid multikey type '$_[0]'";
-	my $n = $mktypes{$key};
-	$n+=128 if $rev eq '-';
-	return $n
-    }
-    die "internal error, bad key '$_[0]'";
-}
-
-our %mkmap = qw(str s
-		string s
-		locale l
-		loc l
-		lstr l
-		int i
-		integer i
-		uint u
-		unsigned_integer u
-		number n
-		num n);
-
-$_ = [$_] for (values %mkmap);
-our %mksub = map { $_ => undef } keys %mkmap;
-
-sub _get_map {
-    my ($rev, $name) = $_[0]=~/^([+-]?)(.*)$/;
-    exists $mkmap{$name}
-	or croak "unknown key type '$name'\n";
-    if ($rev eq '-') {
-	return map { /^-(.*)$/ ? $1 : "-$_" } @{$mkmap{$name}}
-    }
-    @{$mkmap{$name}}
-}
-
-sub _get_sub {
-    $_[0]=~/^[+-]?(.*)$/;
-    exists $mksub{$1}
-	or croak "unknown key type '$1'\n";
-    return $mksub{$1}
-}
-
-sub _combine_map { map { _get_map $_ } @_ }
-
-use constant _nl => "\n";
-
-sub _combine_sub {
-    my $sub = shift;
-    my $for = shift;
-    $for = defined $for ? " for $for" : "";
-
-    my @subs = map { _get_sub $_ } @_;
-
-    if ($sub) {
-	my $code = 'sub { '._nl;
-	if (ref $sub eq 'CODE') {
-	    unless (grep { defined $_ } @subs) {
-		return $sub
-	    }
-	    $code.= 'my @keys = &{$sub};'._nl;
-	}
-	else {
-	    if ($sub eq '@_') {
-		return undef unless grep {defined $_} @subs;
-	    }
-	    $code.= 'my @keys = '.$sub.';'._nl;
-	}
-	$code.= 'print "in: |@keys|\n";'._nl if $DEBUG;
-
-	$code.= '@keys == '.scalar(@_)
-	  . ' or croak "wrong number of keys generated$for '
-	    . '(expected '.scalar(@_).', returned ".scalar(@keys).")";'._nl;
-
-	{ # new scope so @map doesn't get captured
-	    my @map = _combine_map @_;
-	    if (@map==@_) {
-		for my $i (0..$#_) {
-		    if (defined $subs[$i]) {
-			$code.= '{ local $_ = $keys['.$i.']; ($keys['.$i.']) = &{$subs['.$i.']}() }'._nl;
-		    }
-		}
-		$code.='print "out: |@keys|\n";'._nl if $DEBUG;
-		$code.='return @keys'._nl;
-	    }
-	    else {
-		$code.='my @keys1;'._nl;
-		for my $i (0..$#_) {
-		    if (defined $subs[$i]) {
-			$code.= '{ local $_ = shift @keys; push @keys1, &{$subs['.$i.']}() }'._nl;
-		    }
-		    else {
-			$code.= 'push @keys1, shift @keys;'._nl;
-		    }
-		}
-		$code.='print "out: |@keys1|\n";'._nl if $DEBUG;
-		$code.='return @keys1'._nl;
-	    }
-	}
-	$code.='}'._nl;
-	print "CODE$for:\n$code----\n" if $DEBUG >= 2;
-	my $map = eval $code;
-	$@ and die "internal error: code generation failed ($@)";
-	return $map;
-    }
-    else {
-	@_==1 or croak "too many keys or keygen subroutine undefined$for";
-	return @subs;
-    }
-}
-
-sub register_type {
-    my $name = shift;
-    my $sub = shift;
-    $name=~/^\w+(?:::\w+)*$/
-	or croak "invalid type name '$name'";
-    @_ or
-	croak "too few keys";
-    (exists $mkmap{$name} or exists $mktypes{$name})
-	and croak "type '$name' already registered or reserved in ".__PACKAGE__;
-    $mkmap{$name} = [ _combine_map @_ ];
-    $mksub{$name} = _combine_sub $sub, $name, @_;
-}
-
 sub multikeysorter {
-    my @keys = @_;
     if (ref $_[0] eq 'CODE') {
 	my $keygen = shift;
-	my $sub = _combine_sub($keygen, undef, @_);
 	@_ or croak "too few keys";
-	my $ptypes = pack('C*', (map { _mks2n $_ } _combine_map(@_)));
-	# print "type 1\n";
-	# return _multikeysorter($ptypes, $keygen);
+        my $ptypes = Sort::Key::Types::combine_types(@_);
+	my $sub = Sort::Key::Types::combine_sub($keygen, undef, @_);
 	return _multikeysorter($ptypes, $sub, undef);
     }
     else {
-	my $sub = _combine_sub('@_', undef, @_);
 	@_ or croak "too few keys";
-	my $ptypes = pack('C*', (map { _mks2n $_ } _combine_map(@_)));
+        my $ptypes = Sort::Key::Types::combine_types(@_);
+	my $sub = Sort::Key::Types::combine_sub('@_', undef, @_);
 	return _multikeysorter($ptypes, undef, $sub)
     }
 }
@@ -190,19 +54,24 @@ sub multikeysorter {
 sub multikeysorter_inplace {
     if (ref $_[0] eq 'CODE') {
 	my $keygen = shift;
-	my $sub = _combine_sub($keygen, undef, @_);
 	@_ or croak "too few keys";
-	my $ptypes = pack('C*', (map { _mks2n $_ } _combine_map(@_)));
+        my $ptypes = Sort::Key::Types::combine_types(@_);
+	my $sub = Sort::Key::Types::combine_sub($keygen, undef, @_);
 	return _multikeysorter_inplace($ptypes, $sub, undef);
     }
     else {
-	my $sub = _combine_sub('@_', undef, @_);
 	@_ or croak "too few keys";
-	my $ptypes = pack('C*', (map { _mks2n $_ } _combine_map(@_)));
+        my $ptypes = Sort::Key::Types::combine_types(@_);
+	my $sub = Sort::Key::Types::combine_sub('@_', undef, @_);
 	return _multikeysorter_inplace($ptypes, undef, $sub);
     }
 }
 
+sub register_type {
+    warn "Warning, Sort::Key API changed: register_type function has been moved to module Sort::Key::Types";
+    goto &Sort::Key::Types::register_type;
+}
+
 
 1;
 
@@ -412,9 +281,9 @@ Types accepted by default are:
   string, str, locale, loc, integer, int,
   unsigned_integer, uint, number, num
 
-and support for additional types can be added via the non exportable
-L<register_type> subroutine (see below) or the more friendly interface
-available from L<Sort::Key::Register>.
+and support for additional types can be added via the L<register_type>
+subroutine available from L<Sort::Key::Types> or the more
+friendly interface available from L<Sort::Key::Register>.
 
 Types can be preceded by a minus sign to indicate descending order.
 
@@ -431,38 +300,6 @@ Example:
   my @sorted2 = &$sorter2(sub {length $_, $_}, qw(foo fo o of oof));
 
 
-=item Sort::Key::register_type($name, \&gensubkeys, @subkeystypes)
-
-registers a new datatype named C<$name> defining how to convert it to
-a multikey.
-
-C<&gensubkeys> should convert the object of type C<$name> passed on
-C<$_> to a list of values composing the multikey.
-
-C<@subkeystypes> is the list of types for the generated multikeys.
-
-For instance:
-
-  Sort::Key::register_type Person =>
-                 sub { $_->surname,
-                       $_->name,
-                       $_->middlename },
-                 qw(str str str);
-
-  Sort::Key::register_type Color =>
-                 sub { $_->R, $_->G, $_->B },
-                 qw(int int int);
-
-Once a datatype has been registered it can be used in the same way
-as types supported natively, even for defining new types, i.e.:
-
-  Sort::Key::register_type Family =>
-                 sub { $_->man, $_->woman },
-                 qw(Person Person);
-
-
-
-
 =back
 
 
@@ -473,12 +310,35 @@ perl L<sort> function, L<integer>, L<locale>.
 Companion modules L<Sort::Key::Multi>, L<Sort::Key::Register>,
 L<Sort::Key::Maker> and L<Sort::Key::Natural>.
 
-Other interesting Perl sorting modules are L<Sort::Maker> and
-L<Sort::Natural>.
+L<Sort::Key::IPv4>, L<Sort::Key::DateTime> and L<Sort::Key::OID>
+modules add support for additional datatypes to Sort::Key.
+
+L<Sort::Key::External> allows to sort huge lists that do not fit in
+the available memory.
+
+Other interesting Perl sorting modules are L<Sort::Maker>,
+L<Sort::Naturally> and L<Sort::External>.
+
+=head1 SUPPORT
+
+To report bugs, send me and email or use the CPAN bug tracking system
+at L<http://rt.cpan.org>.
+
+=head2 Commercial support
+
+Commercial support, professional services and custom software
+development around this module are available through my current
+company. Drop me an email with a rough description of your
+requirements and we will get back to you ASAP.
+
+=head2 My wishlist
+
+If you like this module and you're feeling generous, take a look at my
+Amazon Wish List: L<http://amzn.com/w/1WU1P6IR5QZ42>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2005, 2006 by Salvador FandiE<ntilde>o,
+Copyright (C) 2005-2007, 2012 by Salvador FandiE<ntilde>o,
 E<lt>sfandino@yahoo.comE<gt>.
 
 This library is free software; you can redistribute it and/or modify
